@@ -1,11 +1,12 @@
 /* Replacement sys/time.h file for building GNU Emacs on the Macintosh.
-   Copyright (C) 2000 Free Software Foundation, Inc.
+   Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005,
+      2006, 2007, 2008  Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
 GNU Emacs is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
+the Free Software Foundation; either version 3, or (at your option)
 any later version.
 
 GNU Emacs is distributed in the hope that it will be useful,
@@ -15,10 +16,10 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Emacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+Boston, MA 02110-1301, USA.  */
 
-/* Contributed by Andrew Choi (akochoi@users.sourceforge.net).  */
+/* Contributed by Andrew Choi (akochoi@mac.com).  */
 
 #ifndef _SYS_TIME_H
 #define _SYS_TIME_H
@@ -28,4 +29,22 @@ struct timeval {
   long tv_usec;  /* microseconds */
 };
 
+#define ITIMER_REAL      0
+#if 0
+#define ITIMER_VIRTUAL   1
+#define ITIMER_PROF      2
+#endif
+
+struct itimerval {
+#if 0
+  struct timeval it_interval;    /* timer interval */
+#endif
+  struct timeval it_value;       /* current value */
+};
+
+extern int setitimer(int, const struct itimerval *, struct itimerval *);
+
 #endif  /* _SYS_TYPES_H */
+
+/* arch-tag: f85ed04d-0e99-4f97-892b-fe029d0e92f9
+   (do not change this comment) */
